% ------------------------------
% Author: Filippo Ferroni, UPF.
% Date  : 7-12-2008
% ------------------------------

% RWM is the Random Walk Metropolis Routine. Iterated a larger number of 
% times, the program generates a series of draws from the target posterior 
% distribution of the set of parameters. The numeber of iterations, 'N', 
% is set in MAIN.m file.
%
% The input are
%    - the set of times series, 'data'.
%    - an initial value for the parameters, 'theta',
%    - a value for the variance matrix of the algorithm, 'VAR'. There is a
%    tuning parameter, 'k', which is set in MAIN.m file,
%    - the type of estimate, which could be '2s', 'lt','hp'  
%    or 'fd'. Where 2 steps = '2s', hp-dsge = 'hp', lt-dsge = 'lt' and  
%    fd-dsge='fd'. The type of estimate is loaded in the MAIN.m file.
%
% The output are 
%    - A matrix 'theta_p' of dimensions N times the number of parameters,
%     where each row of theta_p contains an accepted draw from the algorithm. 
%    - 2 vectors that collect the value of the log likelihood and the log  
%      prior at each draw, 'loglike_p' and 'logprior_p' respectively,  
%    - the acceptance rate, 'acc'.
% All outputs are saved in a .mat file.

% Initial Values for parameters. The dimension of theta depends on the type
% of estimate; for the two step that is equal to the number of DSGE
% structural parameters, whereas for the one step changes according to the
% filter.
if strcmp(nomi.est(e,:),'2s')==1,
    x=nomi.est(e,:);
else
    x=nomi.filt(filo,:);
end
[theta]=initialvalues(dy,x);

% Intial settings:
largo      = length(theta);
theta_p    = zeros(N,largo);
loglike_p  = zeros(N,1);
logprior_p = zeros(N,1);
acc        = 0;
    
% Variance of metropolis-hasting: 
[VAR]=var_rwm(dy,k,x);

% Evaluating the likelihood and prior at the initial value:
loglike   = like(dy,theta,x,struct);
logprior  = prior(dy,theta,x);

t1=now;
for j=1:N,
        thetanew    = theta+normrnd(0,VAR,1,largo);     
        logpriornew = prior(dy,thetanew,x);
        if logpriornew>-Inf
                [loglikenew]=like(dy,thetanew,x,struct);
                if loglikenew == -Inf;
                        R  = 0;
                else 
                        R  = exp(loglikenew+logpriornew-(loglike+logprior));
                end
        else 
            R = 0;        
        end
        if rand<=R 
                loglike  =  loglikenew;
                logprior =  logpriornew;
                theta    =  thetanew;
                acc      =  acc+1;                
        end
    	theta_p(j,:)     =  theta;
	    loglike_p(j)     =  loglike;
	    logprior_p(j)    =  logprior;
end

% Saving rate of acceptance
acc=acc/N*100;
save(['.\POSTERIORS\output\' num2str(nomi.est(e,:)) '_output_' num2str(nomi.filt(filo,:)) '.mat']);
clc    