function [L]=like(dy,theta,x,struct)

% ------------------------------
% Author: Filippo Ferroni, UPF.
% Date  : 7-12-2008
% ------------------------------

% LIKE calculates the Likelihood with the Kalman Filter. The specification of
% the state space for two step is preatty standard. The specification of
% the state space for the one step estimate depends ont he filter. For
% further details see the .pfd file.
% Input  :  - the set of times series, 'dy',
%           - a value for the parameters, 'theta',
%           - the type of estimate, 'x', which could be '2s', 'lt','hp'  
%           or 'fd'. Where 2 steps = '2s', hp-dsge = 'hp', lt-dsge = 'lt'
%           and fd-dsge='fd'.
%           - the number of structural parameters, 'struct'.
% Output : Likelihood, 'L'.

[T,fil]=size(dy);      % fil is the number of observable variables

% Need first to solve the model:
[PP,QQ,RR,SS,NN,PROBLEM]=model(theta);

% If there is no stable solution to the DSGE model, then the likelihood 
% is set to zero.
if PROBLEM==1
        L=-Inf;
        return;
end

if strcmp(x,'2s')==1,
    Sigma=zeros(size(NN,2));
    for pp=1:size(NN,2)
        Sigma(pp,pp)   = theta(struct+pp);
    end
    % starting point of the KF (endogenous + exogenous states):
    xo           = zeros(size(NN,2)+size(PP,2),1);
    % setting the matrices for the state space system:
    F = [PP  QQ; zeros(size(NN,1),size(PP,2))  NN];
    G = [zeros(size(PP,1),size(QQ,2));  eye(size(NN))];
    Hp = [RR  SS];
    H  = Hp';
    inizio=1;
    R=zeros(fil);

elseif strcmp(x,'hp')==1,
    Sigma = zeros(size(dy,2)+size(NN,1));
    % Trend standard deviations:
    for pp=1:fil
        Sigma(pp,pp)=theta(struct+size(NN,1)+pp);
    end
    % Structural standard deviations;
    for ff=1:size(NN,2)
        Sigma(ff+fil,ff+fil)=theta(struct+ff);
    end
    % writting the state-space form matrices
    F = [eye(fil), eye(fil), zeros(fil,size(PP,2)+size(QQ,2));
        zeros(fil), eye(fil), zeros(fil,size(PP,2)+size(QQ,2));
        zeros(size(PP,1),fil), zeros(size(PP,1),fil),   PP,  QQ; 
        zeros(size(NN,1),fil), zeros(size(NN,1),fil), zeros(size(NN,1),size(PP,2)),  NN];
    G = [zeros(fil,fil), zeros(fil,size(NN,2));
         eye(fil,fil), zeros(fil,size(NN,2));
        zeros(size(PP,1),fil), zeros(size(PP,1),size(NN,2));
        zeros(size(NN,1),fil),  eye(size(NN))];
    Hp = [eye(fil), zeros(fil), RR,  SS];
    H  = Hp';
    % Starting value of KF: since the system is not stationary, we can not start
    % using the unconditional moments and we need to pick an arbitrary
    % starting point, the dimensionality is endogenous + exogenous states.
    xo=zeros(2*fil+size(RR,2)+size(SS,2),1);  
    xo(1:fil)=dy(1,:);
    inizio=2;
    R=zeros(fil);
    
elseif strcmp(x,'lt')==1,
    Sigma=zeros(size(NN,2));
    % Structrural standard deviations:
    for ff=1:size(NN,2)
        Sigma(ff,ff)=theta(struct+ff);
    end
    % Trend standard deviations
    R=zeros(fil);
    for rr=1:fil
        R(rr,rr)=theta(struct+size(NN,1)+rr);
    end
    % starting point of the KF (endogenous + exogenous states)
    xo       = zeros(size(NN,2)+size(PP,2),1);
    % I fix the constant in the linear trend specification because they
    % are difficult to estimate; in particular, the CUMSUM statistics for
    % the constants do not converge. I use the first data point as constant
    % value. I tried also mean(dy) or 1/2*(dy(T,:)-dy(1,:)). 
    a = dy(1,:);
    b = theta(struct+size(NN,1)+fil+1:end);
    dy = dy - (kron(a,ones(T,1)) + kron(b,[1:T]'));  
    F = [PP  QQ; zeros(size(NN,1),size(PP,2))  NN];
    G = [zeros(size(PP,1),size(QQ,2));  eye(size(NN))];
    Hp = [RR  SS];
    H  = Hp';
    inizio=1;

elseif strcmp(x,'fd')==1,
    Sigma=zeros(size(NN,2));
    % Structrural standard deviations:
    for ff=1:size(NN,2)
        Sigma(ff,ff)=theta(struct+ff);
    end
    % Trend standard deviations
    R=zeros(fil);
    for rr=1:fil
        R(rr,rr)=theta(struct+size(NN,1)+rr);
    end
    % starting point of the KF (endogenous + exogenous states)
    xo       = zeros(size(NN,2)+size(PP,2),1);
    F = [PP  QQ; zeros(size(NN,1),size(PP,2))  NN];
    G = [zeros(size(PP,1),size(QQ,2));  eye(size(NN))];
    Hp = [RR  SS];
    H  = Hp';
    gamma = theta(struct+size(NN,1)+fil+1:end);
    Gamma = eye(fil);
    % Since inflation is already stationary we do not need to take first
    % difference, Given that inflation is in the fourth column, I do not 
    % take first difference on the fourth column.
    Gamma(4,4) = 0;
    dy = dy(2:end,:) - dy(1:end-1,:) * Gamma - (kron(gamma,ones(T-1,1)));
    inizio=1;
end

% Starting the KF algorithm
xt        = xo;
L         = 0;
V         = G*Sigma*Sigma'*G';


% Starting values: since some of the systems are not stationary, we can not
% start from the unconditional moments and we need to pick an arbitrary
% starting point and consider a 'large' variance-covariance
if max(eig(F))<1,
    exog=size(F,1);
    Sigma     = inv(eye((exog^2))-kron(F,F))*reshape(V,(exog^2),1);
    Sigma     = reshape(Sigma,exog,exog);
else
    Sigma     = 10*eye(size(F,1));
end

for i=inizio:length(dy),
   Omega   = H'*Sigma*H + R*R';
   if det(Omega)==0;
       Omega = eye(fil);
       L=-Inf;
       return;
   end
   Omegai       = inv(Omega);                 
   K            = Sigma*H*Omegai;              % kalman gain
   innov        = (dy(i,:))' - H'*xt;          % y(t) - H'x(t\t-1)
   xt           = F*(xt + K*innov);            % x(t+1\t)  
   Sigma        = F*(Sigma-K*H'*Sigma)*F' + V;
   L            = L + log(det(Omega))+innov'*Omegai*innov;
end
L              = L + fil*T*log(2*pi);
L              = -(0.5*L);

