function [PP,QQ,RR,SS,NN,PROBLEM]=model(theta)

% ------------------------------
% Author: Filippo Ferroni, UPF.
% Date  : 7-12-2008
% ------------------------------

% MODEL rewrites the log linearized equilibrium conditions in matrix form
% and in a way that suits the Uhlig (1998) algorithm, solve.m. The output
% is the solution of the DSGE linearized model in form of matrices. 

% Input  : - a value for the parameters, 'theta'
% Output : - existence of a stable solution to the DSGE model, 'PROBLEM'.
%          - solution of the DSGE model 'PP', 'QQ', 'RR', 'SS', 'NN'. 

% If PROBELM=1 means that there is no stable solution and then PP=QQ=RR=SS=0. 
% Otherwise (PROBELM=0), the stable solution exists and the matrices, PP QQ
% RR SS, are the equilibrium law of motion of the DSGE model, i.e.
%          x(t) = PP x(t-1) + QQ z(t)
%          y(t) = RR x(t-1) + SS z(t).
% where y(t) are the control variables, x(t) the endogenous states and z(t)
% the exogenous states.

% Fixed Parameters:
beta = 0.99;

% Calling the structural parameters from the parameter vector, 
[sigmac,sigman,rhor,rhopi,rhoy,zetap,rhochi,rhoz]=trans(theta);

% Monetary Policy parameters
m1    = (1-rhor)*rhopi*beta;
m2    = (1-rhor)*rhoy*beta;
% Slope of the New Keynesian Phillips curve
kp    = (1-zetap)*(1-beta*zetap)/zetap;

% This model has
% xt = [lambda, mc, mrs, r]
% yt = [y, n, omega, pi]
% zt = [chi, a, er, em]
% for more detail on the model see the .pdf file.

AA = [ -1,  0,  0,  0,
        0,  0,  0,  0,
        0, -1,  0,  0,
       -1,  0,  -1, 0,
        0,  0,  -1, 0,
        0,  0,  0,  -1];
       
BB = [ 0,  0,  0,  0,
       0,  0,  0,  0,
       0,  0,  0,  0,       
       0,  0,  0,  0,
       0,  0,  0,  0,
       0,  0,  0,  rhor];
            
CC = [-sigmac,   0,  0,  0, 
	    -1,   1,  0,  0,
        -1,   1,  1,  0,
         0,  sigman,  0,  0,
         0,  0,    1,  0,
        m2,   0,   0,  m1];
      
DD = zeros(6,4);
DD(1,1)=1;
DD(2,2)=1;
DD(6,3)=1;

FF = [1, 0, 0, 0,
      0, 0, 0, 0];
    
GG = [-1, 0, 0, 1,
       0, kp, 0, 0];
   
HH = zeros(2,4);

JJ = [ 0, 0, 0, -1,  
	   0,  0, 0, beta];

KK = [ 0,  0,  0,  0,  
	   0,  0,  0, -1];

LL = zeros(2,4);

MM = zeros(2,4);
MM(2,4)=kp;

NN = [rhochi, 0, 0, 0,
		 0, rhoz, 0, 0,
		 0, 0, 0, 0,
         0, 0, 0, 0];

% Setting the options for the Uhlig algorithm:
DISPLAY_IMMEDIATELY=0; 
MANUAL_ROOTS=0;		   
IGNORE_VV_SING = 1;    
PROBLEM=0;	
% There is an option to use the QZ-method, due to Sims (1989, 1996).
% You can activate it by setting DO_QZ = 1;
DO_QZ=0;

TOL = .000001;         

% And now the Uhlig Toolkit:
solve;
calc_qrs;
